package com.winhc.bigdata.flink.java.sink;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.hologres.client.HoloClient;
import com.alibaba.hologres.client.exception.HoloClientException;
import com.winhc.bigdata.flink.java.utils.HoloUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import java.sql.Statement;
import java.util.Collection;
import java.util.Set;
import java.util.concurrent.CompletableFuture;

/**
 * @author π
 * @Description:
 * @date 2021/10/19 10:37
 */
public class HoloSqlSink extends RichSinkFunction<JSONObject> {

    static Log log = LogFactory.getLog(HoloSqlSink.class);

    private HoloClient holoClient = null;

    @Override
    public void open(Configuration parameters) throws Exception {
        log.info("init holo..");
        holoClient = HoloUtils.init();
    }

    @Override
    public void invoke(JSONObject jsonObject, Context context) throws Exception {
        try {
            //获取数据中的Key以及Value
            JSONObject data = jsonObject.getJSONObject("data");
            Set<String> keys = data.keySet();
            Collection<Object> values = data.values();
            Collection<Object> primaryKeys = jsonObject.getJSONArray("primaryKeys");
            String tableName = jsonObject.getString("tn");
            String sql = HoloUtils.genInsertSql(tableName, keys, values, primaryKeys);
            CompletableFuture<Boolean> res1 = holoClient.sql(conn -> {
                Boolean resultSet;
                try (Statement stat = conn.createStatement()) {
                    resultSet = stat.execute(sql);
                }
                return resultSet;
            });
        } catch (HoloClientException e) {
            e.printStackTrace();
            System.out.println(e.getMessage());
        }
    }


}
